from enum import Enum


class PlaceNumberTypeCkIds(Enum):
    NotSet = 0

    PlaceNumberOnes = 1
    PlaceNumberTwos = 2
    PlaceNumberThrees = 3
    PlaceNumberFours = 4
    PlaceNumberFives = 5
    PlaceNumberSixes = 6
