from b_clearer_base.b_source.universes.b_clearer_domain_universes import (
    BClearerDomainUniverses,
)


class BClearerRunUniverses(
    BClearerDomainUniverses
):
    def __init__(self):
        pass
