"""Pipeline generator module for creating pipeline structure from configuration."""

# Import all functionality from the more modular template-based generator
from bclearer_core.pipeline_builder.generator_with_templates import (
    PipelineGenerator,
    PipelineUpdater,
    generate_pipeline,
    update_pipeline,
)
